@@ -267,15 +267,17 @@ class Limit:
 class PlayResult:
     """Returned by :func:`chess.engine.EngineProtocol.play()`."""
 
-    def __init__(self, move, ponder, info=None, draw_offered=False):
+    def __init__(self, move, ponder, info=None, *, draw_offered=False, resigned=False):
         self.move = move
         self.ponder = ponder
         self.info = info or {}
         self.draw_offered = draw_offered
+        self.resigned = resigned
 
     def __repr__(self):
-        return "<{} at {:#x} (move={}, ponder={}, info={}, draw_offered={})>".format(
-            type(self).__name__, id(self), self.move, self.ponder, self.info, self.draw_offered)
+        return "<{} at {:#x} (move={}, ponder={}, info={}, draw_offered={}, resigned={})>".format(
+            type(self).__name__, id(self), self.move, self.ponder, self.info,
+            self.draw_offered, self.resigned)
 
 
 class Info(_IntFlag):
@@ -534,12 +536,12 @@ class MockTransport:
 
             if line.startswith("ping ") and self.expected_pings:
                 self.expected_pings -= 1
-                self.protocol.loop.call_soon(lambda: self.protocol.pipe_data_received(1, line.replace("ping ", "pong ").encode("utf-8") + b"\n"))
+                self.protocol.pipe_data_received(1, line.replace("ping ", "pong ").encode("utf-8") + b"\n")
             else:
                 assert self.expectations, "unexpected: {}".format(line)
                 expectation, responses = self.expectations.popleft()
                 assert expectation == line, "expected {}, got: {}".format(expectation, line)
-                self.protocol.loop.call_soon(lambda: self.protocol.pipe_data_received(1, "\n".join(responses).encode("utf-8") + b"\n"))
+                self.protocol.pipe_data_received(1, "\n".join(responses).encode("utf-8") + b"\n")
 
     def get_pid(self):
         return id(self)
@@ -599,9 +601,9 @@ class EngineProtocol(asyncio.SubprocessProtocol, metaclass=abc.ABCMeta):
             line, self.buffer[fd] = self.buffer[fd].split(b"\n", 1)
             line = line.decode("utf-8")
             if fd == 1:
-                self._line_received(line)
+                self.loop.call_soon(self._line_received, line)
             else:
-                self.error_line_received(line)
+                self.loop.call_soon(self.error_line_received, line)
 
     def error_line_received(self, line):
         LOGGER.warning("%s: stderr >> %s", self, line)
@@ -798,9 +800,7 @@ class EngineProtocol(asyncio.SubprocessProtocol, metaclass=abc.ABCMeta):
         popen_args.update(kwargs)
 
         loop = _get_running_loop()
-        transport, protocol = yield from loop.subprocess_exec(cls, *command, **popen_args)
-        yield from protocol.initialize()
-        return transport, protocol
+        return (yield from loop.subprocess_exec(cls, *command, **popen_args))
 
 
 class CommandState(enum.Enum):
@@ -820,17 +820,19 @@ class BaseCommand:
 
     def _engine_terminated(self, engine, code):
         exc = EngineTerminatedError("engine process died unexpectedly (exit code: {})".format(code))
-        self._handle_exception(engine, exc)
-
-        if self.state in [CommandState.Active, CommandState.Cancelling]:
+        if self.state == CommandState.Active:
             self.engine_terminated(engine, exc)
+        elif self.state == CommandState.Cancelling:
+            self.finished.set_result(None)
+        elif self.state == CommandState.New:
+            self._handle_exception(engine, exc)
 
     def _handle_exception(self, engine, exc):
         if not self.result.done():
             self.result.set_exception(exc)
         else:
             self.loop.call_exception_handler({
-                "message": "engine command failed after returning preliminary result",
+                "message": "engine command failed after returning preliminary result ({!r})".format(self.result),
                 "exception": exc,
                 "protocol": engine,
                 "transport": engine.transport,
@@ -884,7 +886,7 @@ class BaseCommand:
         pass
 
     def engine_terminated(self, engine, exc):
-        pass
+        self._handle_exception(engine, exc)
 
     def __repr__(self):
         return "<{} at {:#x} (state={}, result={}, finished={}>".format(type(self).__name__, id(self), self.state, self.result, self.finished)
@@ -901,6 +903,7 @@ class UciProtocol(EngineProtocol):
         super().__init__()
         self.options = UciOptionMap()
         self.config = UciOptionMap()
+        self.target_config = UciOptionMap()
         self.id = {}
         self.board = chess.Board()
         self.game = None
@@ -982,6 +985,9 @@ class UciProtocol(EngineProtocol):
                 option = Option(name, type, without_default.parse(default), min, max, var)
                 engine.options[option.name] = option
 
+                if option.default is not None and not option.is_managed():
+                    engine.target_config[option.name] = option.default
+
             def _id(self, engine, arg):
                 key, value = arg.split(" ", 1)
                 engine.id[key] = value
@@ -997,7 +1003,7 @@ class UciProtocol(EngineProtocol):
 
     def debug(self, on=True):
         """
-        Switches debug move of the engine on or off. This does not interrupt
+        Switches debug mode of the engine on or off. This does not interrupt
         other ongoing operations.
         """
         if on:
@@ -1046,17 +1052,17 @@ class UciProtocol(EngineProtocol):
             self.config[name] = value
 
     def _configure(self, options):
-        for name, value in options.items():
+        for name, value in collections.ChainMap(options, self.target_config).items():
             if name.lower() in MANAGED_OPTIONS:
                 raise EngineError("cannot set {} which is automatically managed".format(name))
-            else:
-                self._setoption(name, value)
+            self._setoption(name, value)
 
     @asyncio.coroutine
     def configure(self, options):
         class Command(BaseCommand):
             def start(self, engine):
                 engine._configure(options)
+                engine.target_config.update({name: value for name, value in options.items() if value is not None})
                 self.set_finished()
 
         return (yield from self.communicate(Command))
@@ -1094,59 +1100,44 @@ class UciProtocol(EngineProtocol):
 
     def _go(self, limit, *, root_moves=None, ponder=False, infinite=False):
         builder = ["go"]
-
         if ponder:
             builder.append("ponder")
-
         if limit.white_clock is not None:
             builder.append("wtime")
             builder.append(str(int(limit.white_clock * 1000)))
-
         if limit.black_clock is not None:
             builder.append("btime")
             builder.append(str(int(limit.black_clock * 1000)))
-
         if limit.white_inc is not None:
             builder.append("winc")
             builder.append(str(int(limit.white_inc * 1000)))
-
         if limit.black_inc is not None:
             builder.append("binc")
             builder.append(str(int(limit.black_inc * 1000)))
-
         if limit.remaining_moves is not None and int(limit.remaining_moves) > 0:
             builder.append("movestogo")
             builder.append(str(int(limit.remaining_moves)))
-
         if limit.depth is not None:
             builder.append("depth")
             builder.append(str(int(limit.depth)))
-
         if limit.nodes is not None:
             builder.append("nodes")
             builder.append(str(int(limit.nodes)))
-
         if limit.mate is not None:
             builder.append("mate")
             builder.append(str(int(limit.mate)))
-
         if limit.time is not None:
             builder.append("movetime")
             builder.append(str(int(limit.time * 1000)))
-
         if infinite:
             builder.append("infinite")
-
         if root_moves:
             builder.append("searchmoves")
             builder.extend(move.uci() for move in root_moves)
-
         self.send_line(" ".join(builder))
 
     @asyncio.coroutine
     def play(self, board, limit, *, game=None, info=INFO_NONE, ponder=False, root_moves=None, options={}):
-        previous_config = self.config.copy()
-
         class Command(BaseCommand):
             def start(self, engine):
                 self.info = {}
@@ -1201,8 +1192,7 @@ class UciProtocol(EngineProtocol):
                             try:
                                 bestmove = engine.board.parse_uci(tokens[0])
                             except ValueError as err:
-                                self.result.set_exception(EngineError(err))
-                                return
+                                raise EngineError(err)
 
                         pondermove = None
                         if bestmove is not None and len(tokens) >= 3 and tokens[1] == "ponder" and tokens[2] != "(none)":
@@ -1212,7 +1202,7 @@ class UciProtocol(EngineProtocol):
                             except ValueError:
                                 LOGGER.exception("engine sent invalid ponder move")
 
-                        self.result.set_result(PlayResult(bestmove, pondermove, self.info, False))
+                        self.result.set_result(PlayResult(bestmove, pondermove, self.info))
 
                         if ponder and pondermove:
                             self.pondering = True
@@ -1223,25 +1213,20 @@ class UciProtocol(EngineProtocol):
                         self.end(engine)
 
             def end(self, engine):
-                # Restore options.
-                for name, value in previous_config.items():
-                    if name.lower() not in MANAGED_OPTIONS:
-                        engine._setoption(name, value)
-                for name, option in engine.options.items():
-                    if name.lower() not in MANAGED_OPTIONS and name not in previous_config and option.default is not None:
-                        engine._setoption(name, option.default)
-
                 self.set_finished()
 
             def cancel(self, engine):
                 engine.send_line("stop")
 
+            def engine_terminated(self, engine, exc):
+                # Allow terminating engine while pondering.
+                if not self.result.done():
+                    super().engine_terminated(engine, exc)
+
         return (yield from self.communicate(Command))
 
     @asyncio.coroutine
     def analysis(self, board, limit=None, *, multipv=None, game=None, info=INFO_ALL, root_moves=None, options={}):
-        previous_config = self.config.copy()
-
         class Command(BaseCommand):
             def start(self, engine):
                 self.analysis = AnalysisResult(stop=lambda: self.cancel(engine))
@@ -1249,7 +1234,6 @@ class UciProtocol(EngineProtocol):
 
                 if "UCI_AnalyseMode" in engine.options:
                     engine._setoption("UCI_AnalyseMode", True)
-
                 if "MultiPV" in engine.options or (multipv and multipv > 1):
                     engine._setoption("MultiPV", 1 if multipv is None else multipv)
 
@@ -1288,14 +1272,6 @@ class UciProtocol(EngineProtocol):
                 self.analysis.post(_parse_uci_info(arg, engine.board, info))
 
             def _bestmove(self, engine, arg):
-                # Restore options.
-                for name, value in previous_config.items():
-                    if name.lower() not in MANAGED_OPTIONS:
-                        engine._setoption(name, value)
-                for name, option in engine.options.items():
-                    if name.lower() not in MANAGED_OPTIONS and name not in previous_config and option.default is not None:
-                        engine._setoption(name, option.default)
-
                 self.analysis.set_finished()
                 self.set_finished()
 
@@ -1500,6 +1476,7 @@ class XBoardProtocol(EngineProtocol):
             "computer": Option("computer", "check", False, None, None, None),
         }
         self.config = {}
+        self.target_config = {}
         self.board = chess.Board()
         self.game = None
         self.first_game = True
@@ -1547,8 +1524,12 @@ class XBoardProtocol(EngineProtocol):
             def end(self, engine):
                 if not engine.features.get("ping", 0):
                     self.result.set_exception(EngineError("xboard engine did not declare required feature: ping"))
+                    self.set_finished()
+                    return
                 if not engine.features.get("setboard", 0):
                     self.result.set_exception(EngineError("xboard engine did not declare required feature: setboard"))
+                    self.set_finished()
+                    return
 
                 if not engine.features.get("reuse", 1):
                     LOGGER.warning("%s: Rejecting feature reuse=0", engine)
@@ -1578,6 +1559,10 @@ class XBoardProtocol(EngineProtocol):
                         engine.options[name] = Option(name, "path", None, None, None, None)
                     engine.send_line("accept egt")
 
+                for option in engine.options.values():
+                    if option.default is not None and not option.is_managed():
+                        engine.target_config[option.name] = option.default
+
                 engine.initialized = True
                 self.set_finished()
 
@@ -1665,17 +1650,15 @@ class XBoardProtocol(EngineProtocol):
 
     @asyncio.coroutine
     def play(self, board, limit, *, game=None, info=INFO_NONE, ponder=False, root_moves=None, options={}):
-        previous_config = self.config.copy()
-
         if root_moves is not None:
-            raise EngineError("play with root_moves, but xboard supports include only in analysis mode")
+            raise EngineError("play with root_moves, but xboard supports 'include' only in analysis mode")
 
         class Command(BaseCommand):
             def start(self, engine):
-                self.info = {}
+                self.play_result = PlayResult(None, None)
                 self.stopped = False
-                self.final_pong = None
-                self.draw_offered = False
+                self.pong_after_move = None
+                self.pong_after_ponder = None
 
                 # Set game, position and configure.
                 engine._new(board, game, options)
@@ -1714,20 +1697,28 @@ class XBoardProtocol(EngineProtocol):
             def line_received(self, engine, line):
                 if line.startswith("move "):
                     self._move(engine, line.split(" ", 1)[1])
-                elif line == self.final_pong:
+                elif line.startswith("Hint: "):
+                    self._hint(engine, line.split(" ", 1)[1])
+                elif line == self.pong_after_move:
                     if not self.result.done():
-                        self.result.set_exception(EngineError("xboard engine answered final pong before sending move"))
-                    self.end(engine)
+                        self.result.set_result(self.play_result)
+                    if not ponder:
+                        self.set_finished()
+                elif line == self.pong_after_ponder:
+                    if not self.result.done():
+                        self.result.set_result(self.play_result)
+                    self.set_finished()
                 elif line == "offer draw":
-                    self.draw_offered = True
-                elif line == "resign":
-                    self.result.set_exception(EngineError("xboard engine resigned"))
-                    self.end(engine)
-                elif line.startswith("1-0") or line.startswith("0-1") or line.startswith("1/2-1/2"):
                     if not self.result.done():
-                        self.result.set_result(PlayResult(None, None, self.info, self.draw_offered))
-                    self.end(engine)
-                elif line.startswith("#") or line.startswith("Hint:"):
+                        self.play_result.draw_offered = True
+                    self._ping_after_move(engine)
+                elif line == "resign":
+                    if not self.result.done():
+                        self.play_result.resigned = True
+                    self._ping_after_move(engine)
+                elif line.startswith("1-0") or line.startswith("0-1") or line.startswith("1/2-1/2"):
+                    self._ping_after_move(engine)
+                elif line.startswith("#"):
                     pass
                 elif len(line.split()) >= 4 and line.lstrip()[0].isdigit():
                     self._post(engine, line)
@@ -1736,19 +1727,36 @@ class XBoardProtocol(EngineProtocol):
 
             def _post(self, engine, line):
                 if not self.result.done():
-                    self.info = _parse_xboard_post(line, engine.board, info)
+                    self.play_result.info = _parse_xboard_post(line, engine.board, info)
 
             def _move(self, engine, arg):
-                if not self.result.cancelled():
+                if not self.result.done() and self.play_result.move is None:
                     try:
-                        move = engine.board.push_xboard(arg)
+                        self.play_result.move = engine.board.push_xboard(arg)
                     except ValueError as err:
                         self.result.set_exception(EngineError(err))
+                    else:
+                        self._ping_after_move(engine)
+                else:
+                    try:
+                        engine.board.push_xboard(arg)
+                    except ValueError:
+                        LOGGER.exception("exception playing unexpected move")
 
-                    self.result.set_result(PlayResult(move, None, self.info, self.draw_offered))
+            def _hint(self, engine, arg):
+                if not self.result.done() and self.play_result.move is not None and self.play_result.ponder is None:
+                    try:
+                        self.play_result.ponder = engine.board.parse_xboard(arg)
+                    except ValueError:
+                        LOGGER.exception("exception parsing hint")
+                else:
+                    LOGGER.warning("unexpected hint: %r", arg)
 
-                if not ponder:
-                    self.end(engine)
+            def _ping_after_move(self, engine):
+                if self.pong_after_move is None:
+                    n = id(self) & 0xffff
+                    self.pong_after_move = "pong {}".format(n)
+                    engine._ping(n)
 
             def cancel(self, engine):
                 if self.stopped:
@@ -1761,25 +1769,19 @@ class XBoardProtocol(EngineProtocol):
                 if ponder:
                     engine.send_line("easy")
 
-                    n = id(self) & 0xffff
-                    self.final_pong = "pong {}".format(n)
+                    n = (id(self) + 1) & 0xffff
+                    self.pong_after_ponder = "pong {}".format(n)
                     engine._ping(n)
 
-            def end(self, engine):
-                if not self.finished.done():
-                    engine._configure(previous_config)
-                    for name, option in engine.options.items():
-                        if name not in previous_config and option.default is not None:
-                            engine._configure({name: option.default})
-
-                    self.set_finished()
+            def engine_terminated(self, engine, exc):
+                # Allow terminating engine while pondering.
+                if not self.result.done():
+                    super().engine_terminated(engine, exc)
 
         return (yield from self.communicate(Command))
 
     @asyncio.coroutine
     def analysis(self, board, limit=None, *, multipv=None, game=None, info=INFO_ALL, root_moves=None, options={}):
-        previous_config = self.config.copy()
-
         if multipv is not None:
             raise EngineError("xboard engine does not support multipv")
 
@@ -1842,12 +1844,6 @@ class XBoardProtocol(EngineProtocol):
                     self.time_limit_handle.cancel()
 
                 self.analysis.set_finished()
-
-                engine._configure(previous_config)
-                for name, option in engine.options.items():
-                    if name not in previous_config and option.default is not None:
-                        engine._configure({name: option.default})
-
                 self.set_finished()
 
             def cancel(self, engine):
@@ -1872,36 +1868,50 @@ class XBoardProtocol(EngineProtocol):
 
         return (yield from self.communicate(Command))
 
+    def _getoption(self, option, default=None):
+        if option in self.config:
+            return self.config[option]
+        if option in self.options:
+            return self.options[option].default
+        return default
+
+    def _setoption(self, name, value):
+        if value is not None and value == self._getoption(name):
+            return
+
+        try:
+            option = self.options[name]
+        except KeyError:
+            raise EngineError("unsupported xboard option or command: {}".format(name))
+
+        self.config[name] = value = option.parse(value)
+
+        if name in ["random", "computer"]:
+            # Applied in _new.
+            pass
+        elif name in ["memory", "cores"] or name.startswith("egtpath "):
+            self.send_line("{} {}".format(name, value))
+        elif value is None:
+            self.send_line("option {}".format(name))
+        elif value is True:
+            self.send_line("option {}=1".format(name))
+        elif value is False:
+            self.send_line("option {}=0".format(name))
+        else:
+            self.send_line("option {}={}".format(name, value))
+
     def _configure(self, options):
-        for name, value in options.items():
-            if value is not None and self.config.get(name) == value:
-                continue
-
-            try:
-                option = self.options[name]
-            except KeyError:
-                raise EngineError("unsupported xboard option or command: {}".format(name))
-
-            self.config[name] = value = option.parse(value)
-
-            if name in ["random", "computer"]:
-                pass
-            elif name in ["memory", "cores"] or name.startswith("egtpath "):
-                self.send_line("{} {}".format(name, value))
-            elif value is None:
-                self.send_line("option {}".format(name))
-            elif value is True:
-                self.send_line("option {}=1".format(name))
-            elif value is False:
-                self.send_line("option {}=0".format(name))
-            else:
-                self.send_line("option {}={}".format(name, value))
+        for name, value in collections.ChainMap(options, self.target_config).items():
+            if name.lower() in MANAGED_OPTIONS:
+                raise EngineError("cannot set {} which is automatically managed".format(name))
+            self._setoption(name, value)
 
     @asyncio.coroutine
     def configure(self, options):
         class Command(BaseCommand):
             def start(self, engine):
                 engine._configure(options)
+                engine.target_config.update({name: value for name, value in options.items() if value is not None})
                 self.set_finished()
 
         return (yield from self.communicate(Command))
@@ -2117,7 +2127,13 @@ def popen_uci(command, *, setpgrp=False, **popen_args):
 
     Returns a subprocess transport and engine protocol pair.
     """
-    return (yield from UciProtocol.popen(command, setpgrp=setpgrp, **popen_args))
+    transport, protocol = yield from UciProtocol.popen(command, setpgrp=setpgrp, **popen_args)
+    try:
+        yield from protocol.initialize()
+    except:
+        transport.close()
+        raise
+    return transport, protocol
 
 
 @asyncio.coroutine
@@ -2137,7 +2153,13 @@ def popen_xboard(command, *, setpgrp=False, **popen_args):
 
     Returns a subprocess transport and engine protocol pair.
     """
-    return (yield from XBoardProtocol.popen(command, setpgrp=setpgrp, **popen_args))
+    transport, protocol = yield from XBoardProtocol.popen(command, setpgrp=setpgrp, **popen_args)
+    try:
+        yield from protocol.initialize()
+    except:
+        transport.close()
+        raise
+    return transport, protocol
 
 
 class SimpleEngine:
@@ -2200,6 +2222,12 @@ class SimpleEngine:
             future = asyncio.run_coroutine_threadsafe(_get(), self.protocol.loop)
         return future.result()
 
+    def communicate(self, command_factory):
+        with self._not_shut_down():
+            coro = self.protocol.communicate(command_factory)
+            future = asyncio.run_coroutine_threadsafe(coro, self.protocol.loop)
+        return future.result()
+
     def configure(self, options):
         with self._not_shut_down():
             coro = asyncio.wait_for(self.protocol.configure(options), self.timeout)
@@ -2252,12 +2280,15 @@ class SimpleEngine:
     def popen(cls, Protocol, command, *, timeout=10.0, debug=False, setpgrp=False, **popen_args):
         @asyncio.coroutine
         def background(future):
-            transport, protocol = yield from asyncio.wait_for(Protocol.popen(command, setpgrp=setpgrp, **popen_args), timeout)
+            transport, protocol = yield from Protocol.popen(command, setpgrp=setpgrp, **popen_args)
             simple_engine = cls(transport, protocol, timeout=timeout)
-            future.set_result(simple_engine)
-            returncode = yield from protocol.returncode
-            simple_engine.returncode.set_result(returncode)
-            simple_engine.close()
+            try:
+                yield from asyncio.wait_for(protocol.initialize(), timeout)
+                future.set_result(simple_engine)
+                returncode = yield from protocol.returncode
+                simple_engine.returncode.set_result(returncode)
+            finally:
+                simple_engine.close()
             yield from simple_engine.shutdown_event.wait()
 
         return run_in_background(background, debug=debug)
