@@ -113,14 +113,19 @@ SKIP = object()
 class GameNode:
     def __init__(self):
         self.parent = None
+        self.variations = []
+
         self.move = None
         self.nags = set()
         self.starting_comment = ""
         self.comment = ""
-        self.variations = []
 
         self.board_cached = None
 
+    @classmethod
+    def dangling_node(cls):
+        return GameNode()
+
     def board(self, *, _cache=True):
         """
         Gets a board with the position of the node.
@@ -265,12 +270,13 @@ class GameNode:
 
     def add_variation(self, move, *, comment="", starting_comment="", nags=()):
         """Creates a child node with the given attributes."""
-        node = GameNode()
+        node = type(self).dangling_node()
         node.move = move
         node.nags = set(nags)
-        node.parent = self
         node.comment = comment
         node.starting_comment = starting_comment
+
+        node.parent = self
         self.variations.append(node)
         return node
 
@@ -392,7 +398,8 @@ class GameNode:
         return self.accept(StringExporter(columns=None))
 
     def __repr__(self):
-        return "<GameNode at {:#x} ({}{} {} ...)>".format(
+        return "<{} at {:#x} ({}{} {} ...)>".format(
+            type(self).__name__,
             id(self),
             self.parent.board().fullmove_number,
             "." if self.parent.board().turn == chess.WHITE else "...",
@@ -490,8 +497,13 @@ class Game(GameNode):
         """Creates an empty game without the default 7 tag roster."""
         return cls(headers={})
 
+    @classmethod
+    def builder(cls):
+        return GameBuilder(Game=cls)
+
     def __repr__(self):
-        return "<Game at {:#x} ({!r} vs. {!r}, {!r})>".format(
+        return "<{} at {:#x} ({!r} vs. {!r}, {!r})>".format(
+            type(self).__name__,
             id(self),
             self.headers.get("White", "?"),
             self.headers.get("Black", "?"),
@@ -588,6 +600,10 @@ class Headers(collections.abc.MutableMapping):
             type(self).__name__,
             ", ".join("{}={!r}".format(key, value) for key, value in self.items()))
 
+    @classmethod
+    def builder(cls):
+        return HeadersBuilder(Headers=lambda: cls({}))
+
 
 class Mainline:
     def __init__(self, start, f=lambda node: node):
@@ -746,13 +762,16 @@ class BaseVisitor:
         raise error
 
 
-class GameCreator(BaseVisitor):
+class GameBuilder(BaseVisitor):
     """
     Creates a game model. Default visitor for :func:`~chess.pgn.read_game()`.
     """
 
+    def __init__(self, *, Game=Game):
+        self.Game = Game
+
     def begin_game(self):
-        self.game = Game()
+        self.game = self.Game()
 
         self.variation_stack = [self.game]
         self.starting_comment = ""
@@ -811,11 +830,14 @@ class GameCreator(BaseVisitor):
         return self.game
 
 
-class HeaderCreator(BaseVisitor):
+class HeadersBuilder(BaseVisitor):
     """Collects headers into a dictionary."""
 
+    def __init__(self, *, Headers=lambda: Headers({})):
+        self.Headers = Headers
+
     def begin_headers(self):
-        self.headers = Headers({})
+        self.headers = self.Headers()
         return self.headers
 
     def visit_header(self, tagname, tagvalue):
@@ -828,7 +850,7 @@ class HeaderCreator(BaseVisitor):
         return self.headers
 
 
-class BoardCreator(BaseVisitor):
+class BoardBuilder(BaseVisitor):
     """
     Returns the final position of the game. The mainline of the game is
     on the move stack.
@@ -1019,7 +1041,7 @@ class FileExporter(StringExporter):
         return self.__repr__()
 
 
-def read_game(handle, *, Visitor=GameCreator):
+def read_game(handle, *, Visitor=GameBuilder):
     """
     Reads a game from a file opened in text mode.
 
@@ -1065,7 +1087,7 @@ def read_game(handle, *, Visitor=GameCreator):
     The parser is relatively forgiving when it comes to errors. It skips over
     tokens it can not parse. Any exceptions are logged and collected in
     :data:`Game.errors <chess.pgn.Game.errors>`. This behavior can be
-    :func:`overriden <chess.pgn.GameCreator.handle_error>`.
+    :func:`overriden <chess.pgn.GameBuilder.handle_error>`.
 
     Returns the parsed game or ``None`` if the end of file is reached.
     """
@@ -1300,7 +1322,7 @@ def read_headers(handle):
     3067
     <Game at ... ('Garry Kasparov' vs. 'Deep Blue (Computer)', 1997.??.??)>
     """
-    return read_game(handle, Visitor=HeaderCreator)
+    return read_game(handle, Visitor=HeadersBuilder)
 
 
 def skip_game(handle):
@@ -1308,3 +1330,9 @@ def skip_game(handle):
     Skip a game. Returns ``True`` if a game was found and skipped.
     """
     return read_game(handle, Visitor=SkipVisitor)
+
+
+# TODO: Remove aliases
+GameCreator = GameBuilder
+HeaderCreator = HeadersBuilder
+BoardCreator = BoardBuilder
